from zope import component

from repoze.bfg.skins.template import SkinApi

from repoze.bfg.htmlpage.template import get_htmlpage
from repoze.bfg.htmlpage.interfaces import IHTMLPage
from chameleon.core.template import Macro

class HTMLPageApi(SkinApi):
    """HTML-page helper."""
    
    def __call__(self, name=None):
        if name is not None:
            htmlpage = get_htmlpage(self.context, self.request, name)
        else:
            htmlpage = component.getMultiAdapter(
                (self.context, self.request), IHTMLPage)

        def render(**kwargs):
            return htmlpage.template.render_macro(
                "", global_scope=False, parameters=kwargs)
        
        return Macro(render)



import os

import zope.interface
import zope.component
import zope.testing

import unittest

OPTIONFLAGS = (zope.testing.doctest.ELLIPSIS |
               zope.testing.doctest.NORMALIZE_WHITESPACE)

import zope.component
import zope.component.testing

import repoze.bfg.htmlpage
import repoze.bfg.htmlpage.tests

def setUp(suite):
    zope.component.testing.setUp(suite)

def test_suite():
    doctests = "zcml.txt", "template.txt",
    
    globs = dict(interface=zope.interface,
                 component=zope.component,
                 path=repoze.bfg.htmlpage.tests.__path__[0],
                 os=os)

    return unittest.TestSuite(
        [zope.testing.doctest.DocFileSuite(
                doctest,
                optionflags=OPTIONFLAGS,
                setUp=setUp,
                globs=globs,
                tearDown=zope.component.testing.tearDown,
                package="repoze.bfg.htmlpage") for doctest in doctests]
        )

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



import os
import interfaces

from zope import interface
from zope import component

from chameleon.html import template

def get_htmlpage(context, request, name):
    """Search directories for a template with the specified
    name. First match is returned."""

    for component_name, directory in component.getAdapters(
        (context, request), interfaces.ITemplateDirectory):
        layout = directory.get(name)
        if layout is not None:
            return layout
        
    raise component.ComponentLookupError(
        "No such template: %s." % repr(name))

class HTMLPage(object):
    interface.implements(interfaces.IHTMLPage)
    
    def __init__(self, filename):
        self.template = template.DynamicHTMLFile(filename)
        self.filename = filename
        self.path = os.path.dirname(filename)
        
    def __repr__(self):
        return '<HTMLPage filename="%s">' % self.template.filename

    @property
    def slots(self):
        return tuple(self.template.parser.slots)

    @property
    def attributes(self):
        return tuple(self.template.parser.attributes)

    def get_resource(self, path):
        try:
            return file(os.path.join(self.path, path))
        except IOError:
            raise IOError(
                "Unable to locate resource %s referenced in layout %s." % (
                repr(path), repr(self.filename)))
            
    def render(self, content, attributes, **kwargs):
        return self.template(content=content, attributes=attributes, **kwargs)

class TemplateDirectory(object):
    interface.implements(interfaces.ITemplateDirectory)

    def __init__(self, path):
        self.path = path
        self.registry = {}
         
    def __repr__(self):
        return '<TemplateDirectory files="%d">' % len(self.registry)

    def __call__(self, context, request):
        """Allows using this class as an adapter."""

        return self
    
    def __getitem__(self, name):
        htmlpage = self.registry.get(name)
        if htmlpage is not None:
            return htmlpage

        # since the layout was not yet initialized, create a new
        # layout (if the filename is found in this directory)
        filename = os.path.join(self.path, "%s.html" % name)
        if os.path.exists(filename):
            htmlpage = HTMLPage(filename)
            self.registry[name] = htmlpage
            return htmlpage
        
        raise KeyError(name)

    def get(self, name, default=None):
        try:
            return self[name]
        except KeyError:
            return default



import os
from setuptools import setup, find_packages, Extension

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup(name='repoze.bfg.htmlpage',
      version = '0.3',
      description='Dynamic HTML pages for repoze.bfg',
      long_description=read('README.txt'),
      keywords = "zope3 repoze bfg",
      classifiers = [
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Zope Public License',
          'Programming Language :: Python',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          'Framework :: Zope3'],
      license='ZPL 2.1',
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages=['repoze', 'repoze.bfg'],
      install_requires=['setuptools',
                        'repoze.bfg',
                        'repoze.bfg.skins',
                        'zope.configuration',
                        'chameleon.html',
                        ],  
      include_package_data = True,
      zip_safe = False,
      )



from zope import component

from zope.configuration import xmlconfig
import zope.configuration.config

from zope.component.zcml import handler
from zope.component.interface import provideInterface
from zope.configuration.exceptions import ConfigurationError
from zope.configuration.fields import GlobalObject, Path
from zope.interface import Interface
from zope.schema import TextLine

from repoze.bfg.interfaces import IRequest

from interfaces import ITemplateDirectory
from template import TemplateDirectory

class IHTMLPageDirective(Interface):
    directory = Path(
        title=u"Directory",
        description=u"""
        Path to the directory containing the templates.""",
        required=True
        )

    for_ = GlobalObject(
        title=u"The interface or class the layouts are for.",
        required=False
        )

    request_type = GlobalObject(
        title=u"""The request type interface for the view""",
        description=(u"The view will be called if the interface represented by "
                     u"'request_type' is implemented by the request.  The "
                     u"default request type is repoze.bfg.interfaces.IRequest"),
        required=False
        )    

def htmlpage(_context, directory, for_=None, request_type=IRequest):
    _context.action(
        discriminator = ('htmlpage', directory, for_, request_type),
        callable = handler,
        args = ('registerAdapter',
                TemplateDirectory(directory),
                (for_, request_type), ITemplateDirectory, directory,
                _context.info),
        )




from zope import interface

class IHTMLPage(interface.Interface):
    slots = interface.Attribute(
        """Slot names.""")

    attributes = interface.Attribute(
        """Element attributes.""")

    def render(content, attributes, **kwargs):
        """Renders the template, inserting content into the slots. The
        ``content`` parameter must be a dictionary mapping slot names
        to unicode strings (document content). To have resources
        automatically rebased, ``context`` and ``request`` must also
        be passed as keyword arguments."""

    def get_resource(path):
        """Return file stream for the resource located at ``path``
        (which is relative to the location of the template file)."""

class ITemplateDirectory(interface.Interface):
    """A template directory represents a file-system directory which
    contains dynamic HTML pages (.html files). This class provides
    dict-like access to them using the template name as key."""

